﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [Item("Transformation", "A transformation applied to a DataAnalysisProblemData")]
  [StorableClass]
  public class DataAnalysisTransformation : ParameterizedNamedItem, IDataAnalysisTransformation {
    #region Parameter Properties
    private IFixedValueParameter<StringValue> OriginalVariableParameter {
      get { return (IFixedValueParameter<StringValue>)Parameters["Original Variable"]; }
    }

    private IFixedValueParameter<StringValue> TransformedVariableParameter {
      get { return (IFixedValueParameter<StringValue>)Parameters["Transformed Variable"]; }
    }

    private IValueParameter<ITransformation> TransformationParameter {
      get { return (IValueParameter<ITransformation>)Parameters["Transformation"]; }
    }
    #endregion

    #region Properties
    public string OriginalVariable {
      get { return OriginalVariableParameter.Value.Value; }
      set { OriginalVariableParameter.Value.Value = value; }
    }

    public string TransformedVariable {
      get { return TransformedVariableParameter.Value.Value; }
      set { TransformedVariableParameter.Value.Value = value; }
    }

    public ITransformation Transformation {
      get { return TransformationParameter.Value; }
      set { TransformationParameter.Value = value; }
    }
    #endregion

    #region Constructor, Cloning & Persistence
    public DataAnalysisTransformation()
      : this("", "", new IdentityTransformation()) {
    }
    public DataAnalysisTransformation(string originalVariable, string transformedVariable, ITransformation transformation)
      : base() {
      Parameters.Add(new FixedValueParameter<StringValue>("Original Variable", new StringValue(originalVariable)));
      Parameters.Add(new FixedValueParameter<StringValue>("Transformed Variable", new StringValue(transformedVariable)));
      Parameters.Add(new ValueParameter<ITransformation>("Transformation", transformation));

      RegisterEventHandlers();
    }

    protected DataAnalysisTransformation(DataAnalysisTransformation original, Cloner cloner)
      : base(original, cloner) {
      RegisterEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DataAnalysisTransformation(this, cloner);
    }

    [StorableConstructor]
    protected DataAnalysisTransformation(bool deserializing)
      : base(deserializing) { }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEventHandlers();
    }
    #endregion

    #region Event-Handling
    private void RegisterEventHandlers() {
      OriginalVariableParameter.Value.ValueChanged += OriginalVariableParameterValue_ValueChanged;
      TransformedVariableParameter.Value.ValueChanged += TransformedVariableParameterValue_ValueChanged;
      TransformationParameter.ValueChanged += TransformationParameter_ValueChanged;
    }

    private void OriginalVariableParameterValue_ValueChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }
    private void TransformedVariableParameterValue_ValueChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }
    private void TransformationParameter_ValueChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }
    #endregion

    public override string ToString() {
      return $"{Transformation} ({OriginalVariable} -> {TransformedVariable})";
    }
  }
}